@@ -1,6 +1,6 @@
 package JE::Undefined;
 
-our $VERSION = '0.007';
+our $VERSION = '0.008';
 
 use strict;
 use warnings;
@@ -42,14 +42,7 @@ Undefined stringifies to 'undefined', and is false as a boolean.
 # A JE::Undefined object is a reference to a global object.
 
 sub new    { bless \do{my $thing = $_[1]}, $_[0] }
-sub prop   { die }
-sub props  { die } # ~~~ implement exception-handling later
-sub delete { die } #     All exceptions that die throw a TypeError
-sub method { die }
 sub value  { undef }
-sub call   { die }
-sub apply  { die }
-sub construct { die }
 sub typeof { 'undefined' }
 sub id     { 'undef' }
 sub primitive { 1 }
